@@ -1,6 +1,8 @@
 pub(crate) mod do_app;
 pub(crate) mod do_build;
 pub(crate) mod do_builder;
+pub(crate) mod do_catalog_schema;
+pub(crate) mod do_catalogs;
 pub(crate) mod do_manifest;
 pub(crate) mod do_namespace;
 pub(crate) mod do_node;
